package com.icod.printer.ui;


import android.Manifest;
import android.app.ProgressDialog;
import android.bluetooth.BluetoothDevice;
import android.content.BroadcastReceiver;
import android.content.Context;
import android.content.ContextWrapper;
import android.content.Intent;
import android.content.IntentFilter;
import android.graphics.Bitmap;
import android.graphics.BitmapFactory;
import android.hardware.usb.UsbDevice;
import android.hardware.usb.UsbManager;
import android.net.Uri;
import android.os.Environment;
import android.os.Handler;
import android.os.Message;
import android.provider.MediaStore;
import android.support.annotation.Nullable;
import android.text.TextUtils;
import android.util.Log;
import android.view.View;
import android.widget.AdapterView;
import android.widget.ArrayAdapter;
import android.widget.Button;
import android.widget.EditText;
import android.widget.LinearLayout;
import android.widget.Spinner;
import android.widget.TextView;

import com.blankj.utilcode.util.CrashUtils;
import com.icod.printer.R;
import com.icod.printer.base.BaseActivity;
import com.icod.printer.utils.DataUtils;
import com.icod.printer.utils.PermissionUtil;
import com.icod.printer.utils.PrintImageUtils;
import com.icod.printer.utils.PrinterCommodityUtils;
import com.icod.printer.utils.StatusDescribe;
import com.icod.serial.SerialPortFinder;
import com.szsicod.print.escpos.PrinterAPI;
import com.szsicod.print.io.BluetoothAPI;
import com.szsicod.print.io.InterfaceAPI;
import com.szsicod.print.io.SerialAPI;
import com.szsicod.print.io.SocketAPI;
import com.szsicod.print.io.USBAPI;
import com.szsicod.print.io.UsbNativeAPI;
import com.szsicod.print.utils.BitmapUtils;

import java.io.File;
import java.io.IOException;
import java.io.InputStream;
import java.io.UnsupportedEncodingException;
import java.util.ArrayList;
import java.util.List;


public class MainActivity extends BaseActivity implements View.OnClickListener {
    private static final String TAG = "MainActivity";
    private static final int DISCONNECT = -5;           // 断开连接
    private static final int NOCONNECT = -6;            // 未连接
    private static final int TOAST_CODE = 10001;        // 吐司弹出
    private static final int TOAST_SHOW_CODE = 10002;        // 吐司弹出
    private static final int REQUEST_CODE_INTENT = 10002;
    private static final int REQUEST_CODE_PICK_IMAGE = 10003;

    // View
    private Button btnConnect, btnDisconnect;
    private Spinner spConnectType;
    private LinearLayout llUSB, llSerial, llWifi;
    private TextView tvUSB;
    private Spinner spDeviceList, spBaudRate, spFlowcontrol;
    private EditText etIp, etPort;
    private Button btnPrintText, btnPrintBarcode, btnPrintQrcode, btnPrintImage, btnPrintCashbox, btnPrintState, btnUpgradeFirmware, btnTestPage, btnTestPrint;
    private TextView tvPrintShow;

    private UsbBroadCastReceiver mUsbBroadCastReceiver;
    private PrinterAPI mPrinter;
    private Runnable runnable;

    private DataUtils dataUtils;
    private String[] connect_type_string;
    private String[] mSerialportList;
    private String[] baud_rate_string;
    private int[] baud_rate_int;
    private String[] serial_flow;

    private Bitmap bitmap;

    private Boolean isStart = false;

    static {
        //如果使用UsbNativeAPI 必须加载 适用root板 这个不需要权限窗口和申请权限
        System.loadLibrary("usb1.0");
        //串口
        System.loadLibrary("serial_icod");
        //图片
        System.loadLibrary("image_icod");
    }

    @Override
    protected int getLayoutId() {
        return R.layout.activity_main;
    }

    // 打印机的连接
    private void funcPrinterConnect() {
        runnable = new Runnable() {
            @Override
            public void run() {
                if (mPrinter.isConnect()) {
                    mPrinter.disconnect();
                    handler.obtainMessage(DISCONNECT).sendToTarget();
                }
                InterfaceAPI io = null;
                switch (spConnectType.getSelectedItemPosition()) {
                    case 0:
                        // USB
                        io = new USBAPI(MainActivity.this);
                        break;
                    case 1:
                        // Bluetooth
                        Intent intent = new Intent(MainActivity.this, BluetoothActivity.class);
                        startActivityForResult(intent, REQUEST_CODE_INTENT);
                        break;
                    case 2:
                        // Serial
                        io = new SerialAPI(new File(mSerialportList[spDeviceList.getSelectedItemPosition()]),
                                baud_rate_int[spBaudRate.getSelectedItemPosition()],
                                spFlowcontrol.getSelectedItemPosition());
                        break;
                    case 3:
                        // Wifi
                        String etIpStr = etIp.getText().toString().trim();
                        String etPortStr = etPort.getText().toString().trim();
                        if (TextUtils.isEmpty(etIpStr) || TextUtils.isEmpty(etPortStr))
                            handler.obtainMessage(TOAST_CODE, getString(R.string.str_Error) + getString(R.string.str_IpPort_Null)).sendToTarget();
                        else
                            io = new SocketAPI(etIpStr, Integer.valueOf(etPortStr));
                        break;
                    case 4:
                        // UsbNative
                        // if your Android mainboard have root permission you can usb this;
                        io = new UsbNativeAPI();
                        break;
                }

                if (io != null) {
                    int ret = mPrinter.connect(io);
                    handler.obtainMessage(ret).sendToTarget();
                }
            }
        };
        new Thread(runnable).start();
    }

    // 打印机断开连接
    private void funcPrinterDisConnect() {
        runnable = new Runnable() {
            @Override
            public void run() {
                try {
                    // 打印方法：disconnect
                    // 打印例范文本

                    int ret = mPrinter.disconnect();
                    handler.obtainMessage(TOAST_CODE, ret == PrinterAPI.SUCCESS ? getString(R.string.str_Connect_Cancel) : getString(R.string.str_No_Connect)).sendToTarget();

                } catch (Exception e) {
                    e.printStackTrace();
                }

            }
        };
        new Thread(runnable).start();
    }


    // 打印文本
    private void funcPrintText() {
        runnable = new Runnable() {
            @Override
            public void run() {
                try {
                    // 打印方法：printString
                    // 打印例范文本

                    String str = "深圳研科深圳研科深圳研科\n" +
                            "深圳研科\n" +
                            "深圳研科\n" +
                            "深圳研科\n" +
                            "深圳研科\n" +
                            "深圳研科\n" +
                            "深圳研科\n" +
                            "123456789\n" +
                            "123456789\n" +
                            "123456789\n" +
                            "123456789\n";

                    mPrinter.printString("文本测试打印:\n");
                    mPrinter.printFeed();
                    mPrinter.printString(str, "GBK", true);
                    mPrinter.printFeed();
                    int ret = mPrinter.cutPaper(66, 0);
                    handler.obtainMessage(TOAST_CODE, ret == PrinterAPI.SUCCESS ? getString(R.string.str_Func_Success) : getString(R.string.str_Func_Failed)).sendToTarget();
                } catch (Exception e) {
                    e.printStackTrace();
                }

            }
        }

        ;
        new Thread(runnable).start();
    }

    // 打印一维码
    private void funcPrintBarcode() {
        runnable = new Runnable() {
            @Override
            public void run() {
                // 打印方法：printBarCode 参数说明查看文档
                // 打印例范文本
                try {

                    mPrinter.printString("条形码测试打印:\n");
                    mPrinter.printFeed();
                    // 条码内容在下方
                    mPrinter.sendOrder(new byte[]{0x1d, 0x48, 0x02});
                    mPrinter.setBarCodeWidth(3);
                    mPrinter.setBarCodeHeight(120);

                    String barStr = "1234567890";
                    mPrinter.printBarCode(73, barStr.length(), barStr);
//                    mPrinter.sendOrder(gainBarCode128(barStr, "C"));

                    mPrinter.printFeed();
                    mPrinter.printString("测试完成！\n");
                    mPrinter.printFeed();

                    int ret = mPrinter.cutPaper(66, 0);
                    handler.obtainMessage(TOAST_CODE, ret == PrinterAPI.SUCCESS ? getString(R.string.str_Func_Success) : getString(R.string.str_Func_Failed)).sendToTarget();
                } catch (UnsupportedEncodingException e) {
                    e.printStackTrace();
                }
            }
        };
        new Thread(runnable).start();
    }

    // 打印二维码
    private void funcPrintQrcode() {
        runnable = new Runnable() {
            @Override
            public void run() {
                // 打印方法：printQRCode 参数说明查看文档
                // 打印例范文本

                try {
                    String qrStr = "http://www.szicod.com/";

                    mPrinter.printString("二维码测试打印:\n");
                    mPrinter.printFeed();
                    mPrinter.printQRCode(qrStr, 10, false);
                    mPrinter.printFeed();
                    mPrinter.printString("测试完成！\n");
                    mPrinter.printFeed();
                    int ret = mPrinter.cutPaper(66, 0);
                    handler.obtainMessage(TOAST_CODE, ret == PrinterAPI.SUCCESS ? getString(R.string.str_Func_Success) : getString(R.string.str_Func_Failed)).sendToTarget();
                } catch (Exception e) {
                    e.printStackTrace();
                }
            }
        };
        new Thread(runnable).start();
    }


    // 打印图片
    private void funcPrintImage(Bitmap bmp) {
        runnable = new Runnable() {
            @Override
            public void run() {
                // 打印方法：printRasterBitmap
                // 打印例范文本
                try {
                    mPrinter.printString("图片测试打印:\n");
                    mPrinter.printFeed();
                    mPrinter.printRasterBitmap(bmp);

//                    byte[] bmpBytes = PrintImageUtils.parseBmpToByte(bmp);
//                    mPrinter.sendOrder(bmpBytes);
                    mPrinter.printFeed();
                    mPrinter.printString("测试完成！\n");
                    mPrinter.printFeed();

                    int ret = mPrinter.cutPaper(66, 0);
                    handler.obtainMessage(TOAST_CODE, ret == PrinterAPI.SUCCESS ? getString(R.string.str_Func_Success) : getString(R.string.str_Func_Failed)).sendToTarget();
                } catch (Exception e) {
                    e.printStackTrace();
                }
            }
        };
        new Thread(runnable).start();
    }

    // 开启钱箱
    private void funcPrintCashbox() {
        runnable = new Runnable() {
            @Override
            public void run() {
                // 打印方法：openCashDrawer
                // 打印例范文本
                try {
                    int ret = mPrinter.openCashDrawer(0, 100, 200);
                    handler.obtainMessage(TOAST_CODE, ret == PrinterAPI.SUCCESS ? getString(R.string.str_Func_Success) : getString(R.string.str_Func_Failed)).sendToTarget();

                } catch (Exception e) {
                    e.printStackTrace();
                }
            }
        };
        new Thread(runnable).start();
    }

    // 查询打印机状态
    private void funcPrintState() {
        runnable = new Runnable() {
            @Override
            public void run() {
                // 打印方法：getStatus
                // 打印例范文本
                try {
                    // Writing a
                    String stateStr = StatusDescribe.getStatusDescribe(mPrinter.getStatus());
                    handler.obtainMessage(TOAST_CODE, getString(R.string.str_Status) + stateStr).sendToTarget();

//                    // Writing b
//                    byte[] bytes = new byte[]{0x10, 0x04, 0x01, 0x10, 0x04, 0x02, 0x10, 0x04, 0x03, 0x10, 0x04, 0x04 };
//                    mPrinter.sendOrder(bytes);
//                    Thread.sleep(200);
//                    byte[] ret = new byte[4];
//                    mPrinter.readIO(ret, 0, ret.length, 1 * 1000);
//                    handler.obtainMessage(TOAST_CODE, getString(R.string.str_Status) + dataUtils.bytesToHexString(ret)).sendToTarget();
                } catch (Exception e) {
                    e.printStackTrace();
                }

            }
        };
        new Thread(runnable).start();
    }

    // 自检页
    private void funcTestPage() {
        runnable = new Runnable() {
            @Override
            public void run() {
                try {
                    int ret = mPrinter.selfTestPage();

                    handler.obtainMessage(TOAST_CODE, ret == PrinterAPI.SUCCESS ? getString(R.string.str_Func_Success) : getString(R.string.str_Func_Failed)).sendToTarget();
                } catch (Exception e) {
                    e.printStackTrace();
                }
            }
        };
        new Thread(runnable).start();
    }

    // 测试打印
    private void funcTestPrint() {
        runnable = new Runnable() {
            @Override
            public void run() {
                try {
                    byte[] aa = PrinterCommodityUtils.readAssetsText(getAssets(), "nihao.txt");
                    int ret = mPrinter.sendOrder(aa);


                    handler.obtainMessage(TOAST_CODE, ret == PrinterAPI.SUCCESS ? getString(R.string.str_Func_Success) : getString(R.string.str_Func_Failed)).sendToTarget();
                } catch (Exception e) {
                    e.printStackTrace();
                }
            }
        };
        new Thread(runnable).start();
    }


    /**
     * 获取条码指令
     *
     * @param barStr 条码内容
     * @param code   条码类型 {A,B,C}
     * @return
     */
    private byte[] gainBarCode128(String barStr, String code) {
        byte[] barStrBytes = barStr.getBytes();
        List<Integer> mCmd = new ArrayList<>();

        mCmd.add(0x1D);
        mCmd.add(0x6B);
        mCmd.add(0x49);
        switch (code) {
            case "C":
            case "c":
                if (barStrBytes.length % 2 == 0) {
                    mCmd.add((barStrBytes.length / 2 + 2));
                    mCmd.add(0x7B);
                    mCmd.add(0x43);
                    for (int i = 0; i < barStrBytes.length / 2; i++) {
                        mCmd.add(Integer.parseInt(String.valueOf(barStr.charAt(2 * i)) + barStr.charAt(2 * i + 1)));
                    }
                } else {
                    mCmd.add((barStr.length() / 2 + 2 + 3));
                    mCmd.add(0x7B);
                    mCmd.add(0x43);
                    for (int i = 0; i < barStr.length() / 2; i++) {
                        mCmd.add(Integer.parseInt(String.valueOf(barStr.charAt(2 * i)) + barStr.charAt(2 * i + 1)));
                    }
                    mCmd.add(0x7B);
                    mCmd.add(0x41);
                    mCmd.add(barStr.charAt(barStr.length() - 1) & 0xff);
                }
                break;
            case "B":
            case "b":
                mCmd.add((barStr.length() + 2));
                mCmd.add(0x7B);
                mCmd.add(0x42);
                for (int i = 0; i < barStr.length(); i++) {
                    mCmd.add(barStr.charAt(i) & 0xff);
                }
                break;
            case "A":
            case "a":
            default:
                mCmd.add((barStr.length() + 2));
                mCmd.add(0x7B);
                mCmd.add(0x41);
                for (int i = 0; i < barStr.length(); i++) {
                    mCmd.add(barStr.charAt(i) & 0xff);
                }
                break;
        }

        byte[] barBytes = new byte[mCmd.size()];
        for (int i = 0; i < mCmd.size(); i++)
            barBytes[i] = (byte) (mCmd.get(i) & 0xff);

        return barBytes;
    }

    @Override
    protected void initView() {
        btnConnect = findViewById(R.id.btn_connect);
        btnDisconnect = findViewById(R.id.btn_disconnect);
        spConnectType = findViewById(R.id.sp_connect_type);
        llUSB = findViewById(R.id.ll_usb);
        llSerial = findViewById(R.id.ll_serial);
        llWifi = findViewById(R.id.ll_wifi);
        tvUSB = findViewById(R.id.tv_usb);
        spDeviceList = findViewById(R.id.sp_device_list);
        spBaudRate = findViewById(R.id.sp_baud_rate);
        spFlowcontrol = findViewById(R.id.sp_flowcontrol);
        etIp = findViewById(R.id.et_ip);
        etPort = findViewById(R.id.et_port);
        btnPrintText = findViewById(R.id.btn_print_text);
        btnPrintBarcode = findViewById(R.id.btn_print_barcode);
        btnPrintQrcode = findViewById(R.id.btn_print_qrcode);
        btnPrintImage = findViewById(R.id.btn_print_image);
        btnPrintCashbox = findViewById(R.id.btn_print_cashbox);
        btnPrintState = findViewById(R.id.btn_print_state);
        btnUpgradeFirmware = findViewById(R.id.btn_upgrade_firmware);
        btnTestPage = findViewById(R.id.btn_test_page);
        btnTestPrint = findViewById(R.id.btn_test_print);

        tvPrintShow = findViewById(R.id.tv_print_show);

        btnConnect.setOnClickListener(this);
        btnDisconnect.setOnClickListener(this);
        tvUSB.setOnClickListener(this);
        btnPrintText.setOnClickListener(this);
        btnPrintBarcode.setOnClickListener(this);
        btnPrintQrcode.setOnClickListener(this);
        btnPrintImage.setOnClickListener(this);
        btnPrintCashbox.setOnClickListener(this);
        btnPrintState.setOnClickListener(this);
        btnUpgradeFirmware.setOnClickListener(this);
        btnTestPage.setOnClickListener(this);
        btnTestPrint.setOnClickListener(this);

//        int printWidth = (80 - 8) * 8;
        int printWidth = (58 - 10) * 8;
        bitmap = BitmapFactory.decodeResource(getResources(), R.mipmap.icod_2);
        bitmap = BitmapUtils.reSize(bitmap, printWidth, bitmap.getHeight() * printWidth / bitmap.getWidth());

        spConnectType.setOnItemSelectedListener(new AdapterView.OnItemSelectedListener() {
            @Override
            public void onItemSelected(AdapterView<?> parent, View view, int position, long id) {
                String sInfo = parent.getItemAtPosition(position).toString();
                if (connect_type_string[0].equals(sInfo)) {
                    if (mUsbBroadCastReceiver == null) {
                        IntentFilter intentFilter = new IntentFilter();
                        intentFilter.addAction(UsbManager.ACTION_USB_DEVICE_DETACHED);
                        intentFilter.addAction(UsbManager.ACTION_USB_DEVICE_ATTACHED);
                        mUsbBroadCastReceiver = new UsbBroadCastReceiver();
                        registerReceiver(mUsbBroadCastReceiver, intentFilter);
                    }
                    llUSB.setVisibility(View.VISIBLE);
                    llSerial.setVisibility(View.GONE);
                    llWifi.setVisibility(View.GONE);
                } else if (connect_type_string[1].equals(sInfo)) {
                    llUSB.setVisibility(View.GONE);
                    llSerial.setVisibility(View.GONE);
                    llWifi.setVisibility(View.GONE);
                } else if (connect_type_string[2].equals(sInfo)) {
                    llUSB.setVisibility(View.GONE);
                    llSerial.setVisibility(View.VISIBLE);
                    llWifi.setVisibility(View.GONE);

                    SerialPortFinder spf = new SerialPortFinder();
                    mSerialportList = spf.getAllDevicesPath();
                    if (mSerialportList.length <= 0) mSerialportList = new String[]{"NONE"};
                    ArrayAdapter<String> adapterTemp = new ArrayAdapter<>(MainActivity.this, android.R.layout.simple_spinner_item, mSerialportList);
                    adapterTemp.setDropDownViewResource(android.R.layout.simple_spinner_dropdown_item);
                    spDeviceList.setAdapter(adapterTemp);

                    ArrayAdapter<String> adapterTemp2 = new ArrayAdapter<>(MainActivity.this, android.R.layout.simple_spinner_item, baud_rate_string);
                    adapterTemp2.setDropDownViewResource(android.R.layout.simple_spinner_dropdown_item);
                    spBaudRate.setAdapter(adapterTemp2);

                    ArrayAdapter<String> adapterTemp3 = new ArrayAdapter<>(MainActivity.this, android.R.layout.simple_spinner_item, serial_flow);
                    adapterTemp3.setDropDownViewResource(android.R.layout.simple_spinner_dropdown_item);
                    spFlowcontrol.setAdapter(adapterTemp3);
                } else if (connect_type_string[3].equals(sInfo)) {
                    llUSB.setVisibility(View.GONE);
                    llSerial.setVisibility(View.GONE);
                    llWifi.setVisibility(View.VISIBLE);

                }
            }

            @Override
            public void onNothingSelected(AdapterView<?> parent) {

            }
        });
    }

    @Override
    protected void initData() {
        mPrinter = PrinterAPI.getInstance();

        dataUtils = DataUtils.getInstance(this);
        connect_type_string = dataUtils.getStringArr(R.array.connect_type_string);
        mSerialportList = dataUtils.getStringArr(R.array.serial_port_list);
        baud_rate_string = dataUtils.getStringArr(R.array.baud_rate_string);
        baud_rate_int = dataUtils.getIntegerArr(R.array.baud_rate);
        serial_flow = dataUtils.getStringArr(R.array.serial_flow);


        ArrayAdapter<String> adapterTemp = new ArrayAdapter<>(this, android.R.layout.simple_spinner_item, connect_type_string);
        adapterTemp.setDropDownViewResource(android.R.layout.simple_spinner_dropdown_item);
        spConnectType.setAdapter(adapterTemp);
        initPermission();

        CrashUtils.init(Environment.getExternalStorageDirectory().toString() + File.separator + "icod" + File.separator);


    }

    @Override
    public void onClick(View v) {
        switch (v.getId()) {
            case R.id.btn_connect:
                funcPrinterConnect();
                break;
            case R.id.btn_disconnect:
                funcPrinterDisConnect();
                break;
            case R.id.tv_usb:
                startActivity(new Intent(MainActivity.this, USBActivity.class));
                break;
            case R.id.btn_print_text:
                funcPrintText();
                break;
            case R.id.btn_print_barcode:
                funcPrintBarcode();
                break;
            case R.id.btn_print_qrcode:
                funcPrintQrcode();
                break;
            case R.id.btn_print_image:
                funcPrintImage(bitmap);
//                Intent intent = new Intent(Intent.ACTION_PICK, MediaStore.Images.Media.EXTERNAL_CONTENT_URI);
//                startActivityForResult(intent, REQUEST_CODE_PICK_IMAGE);
                break;
            case R.id.btn_print_cashbox:
                funcPrintCashbox();
                break;
            case R.id.btn_print_state:
                funcPrintState();
                break;
            case R.id.btn_upgrade_firmware:
                onProgressRun(getString(R.string.str_In_Operation), ProgressDialog.STYLE_HORIZONTAL);
                Update(" ");
                break;
            case R.id.btn_test_page:
                funcTestPage();
                break;
            case R.id.btn_test_print:
                funcTestPrint();
                break;
        }
    }


    //usb 插拔监听
    private class UsbBroadCastReceiver extends BroadcastReceiver {
        @Override
        public void onReceive(Context context, Intent intent) {
            UsbDevice device = (UsbDevice) intent.getParcelableExtra(UsbManager.EXTRA_DEVICE);
            switch (intent.getAction()) {
                case UsbManager.ACTION_USB_DEVICE_ATTACHED:
                    // 当USB设备连接到USB总线时，在主机模式下发送此意图。
                    onToast("插入设备 vid:" + device.getVendorId() + "  pid:" + device.getProductId());
                    break;
                case UsbManager.ACTION_USB_DEVICE_DETACHED:
                    // 当USB设备在主机模式下脱离USB总线时发送此意图。
                    onToast("拔出设备 vid:" + device.getVendorId() + "  pid:" + device.getProductId());
                    break;
            }
        }

    }

    @Override
    protected void onActivityResult(int requestCode, int resultCode, @Nullable Intent data) {
        super.onActivityResult(requestCode, resultCode, data);
        if (data != null)
            if (resultCode == BluetoothActivity.RESULTCODE_CODE_INTENT && requestCode == REQUEST_CODE_INTENT) {
                // 蓝牙返回
                BluetoothDevice bluetoothDevice = data.getParcelableExtra("device");
                Log.d(TAG, "Device: " + bluetoothDevice.getName() + " - " + bluetoothDevice.getAddress());
                onProgressRun(getString(R.string.str_In_Operation));
                runnable = new Runnable() {
                    @Override
                    public void run() {
                        BluetoothAPI mBluetoothApi = new BluetoothAPI(MainActivity.this);
                        mBluetoothApi.setPin("0000");//我厂自动配对的;
                        // A8:18:00:05:1B:97
                        mBluetoothApi.checkDevice(bluetoothDevice.getAddress());
                        int ret = mPrinter.connect(mBluetoothApi);

                        handler.obtainMessage(ret).sendToTarget();
                    }
                };
                new Thread(runnable).start();
            } else if (resultCode != 0 && requestCode == REQUEST_CODE_PICK_IMAGE) {
                // 相册返回
                try {
                    Bitmap photoBmp;
                    Uri uri = data.getData();
                    if (uri == null) {
                        funcPrintImage(bitmap);
                    } else {
                        photoBmp = MediaStore.Images.Media.getBitmap(new ContextWrapper(this).getContentResolver(), uri);
                        if (photoBmp == null) {
                            funcPrintImage(bitmap);
                        } else {
                            funcPrintImage(photoBmp);
                        }
                    }
                } catch (IOException e) {
                    e.printStackTrace();
                }
            }
    }

    public Handler handler = new Handler(new Handler.Callback() {

        @Override
        public boolean handleMessage(Message msg) {
            switch (msg.what) {
                case PrinterAPI.SUCCESS:
                    tvPrintShow.setText(String.format("%s%s", getString(R.string.str_Connect_Printer), getString(R.string.str_Connect_Succeed)));
                    onDismissProgress();
                    break;
                case PrinterAPI.FAIL:
                case PrinterAPI.ERR_PARAM:
                    tvPrintShow.setText(String.format("%s%s", getString(R.string.str_Connect_Printer), getString(R.string.str_Connect_Fail)));
                    onDismissProgress();
                    break;
                case DISCONNECT:
                    tvPrintShow.setText(String.format("%s%s", getString(R.string.str_Connect_Printer), getString(R.string.str_Connect_Cancel)));
                    onDismissProgress();
                    break;
                case NOCONNECT:
                    tvPrintShow.setText(String.format("%s%s", getString(R.string.str_Connect_Printer), getString(R.string.str_No_Connect)));
                    onDismissProgress();
                    break;
                case TOAST_CODE:
                    tvPrintShow.setText((String) msg.obj);
                    break;
                case TOAST_SHOW_CODE:
                    onToast((String) msg.obj);
                    onDismissProgress();
                    break;
            }
            return false;
        }


    });

    /**
     * 固件升级
     *
     * @param fileName assets内的固件名称
     */
    private void Update(String fileName) {
        runnable = () -> {
            try {
                InputStream open = getAssets().open(fileName);
                mPrinter.setmFirmwareListener((progress, code) -> {
                    switch (code) {
                        case PrinterAPI.SUCCESS:
                            progressDialog.setProgress(progress);
                            if (progress == 100) {
                                if (progressDialog != null && progressDialog.isShowing()) {
                                    onDismissProgress();
                                    handler.obtainMessage(TOAST_CODE, getString(R.string.str_Func_Success)).sendToTarget();
                                }
                            }
                            break;
                        case PrinterAPI.FAIL:
                        case PrinterAPI.ERR_PARAM:
                            onDismissProgress();
                            handler.obtainMessage(TOAST_CODE, getString(R.string.str_Func_Failed)).sendToTarget();
                            break;
                    }
                });
                mPrinter.updateFirmware(open);
                open.close();
            } catch (IOException e) {
                e.printStackTrace();
            }

        };
        new Thread(runnable).start();
    }

    private void initPermission() {
        String[] permissionArray = new String[]{Manifest.permission.WRITE_EXTERNAL_STORAGE, Manifest.permission.READ_EXTERNAL_STORAGE, Manifest.permission.ACCESS_COARSE_LOCATION};
        String[] noCheckPermissionArray = PermissionUtil.checkPermissions(this, permissionArray);
        if (noCheckPermissionArray != null && noCheckPermissionArray.length > 0) {
            PermissionUtil.applyPermission(this, noCheckPermissionArray);
        }
    }

    protected void onDestroy() {
        if (mUsbBroadCastReceiver != null)
            unregisterReceiver(mUsbBroadCastReceiver);

        mPrinter.disconnect();

        super.onDestroy();
    }
}
