package com.icod.printer.ui;

import android.content.BroadcastReceiver;
import android.content.Context;
import android.content.Intent;
import android.content.IntentFilter;
import android.graphics.Bitmap;
import android.graphics.BitmapFactory;
import android.hardware.usb.UsbDevice;
import android.hardware.usb.UsbManager;
import android.support.v7.widget.DividerItemDecoration;
import android.support.v7.widget.LinearLayoutManager;
import android.support.v7.widget.RecyclerView;
import android.util.Log;
import android.view.View;

import com.icod.printer.R;
import com.icod.printer.adapter.UsbDeviceAdapter;
import com.icod.printer.base.BaseActivity;
import com.icod.printer.utils.PrintImageUtils;
import com.szsicod.print.escpos.PrinterAPI;
import com.szsicod.print.io.USBAPI;
import com.szsicod.print.utils.BitmapUtils;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;

/**
 * @User Administrator
 * @Date 2021-03-10 17:17
 */
public class USBActivity extends BaseActivity {
    private static final String TAG = "USBActivity";

    private RecyclerView rlDevice;

    private UsbManager mUsbManager;
    private List<UsbDevice> deviceList = new ArrayList<>();
    private UsbDeviceAdapter usbDeviceAdapter;

    private UsbBroadCastReceiver mUsbBroadCastReceiver;
    private List<PrinterAPI> printerArray = new ArrayList<>();

    @Override
    protected int getLayoutId() {
        return R.layout.usb_main;
    }

    @Override
    protected void initView() {
        rlDevice = findViewById(R.id.rcv_usb);

        findViewById(R.id.btn_Search).setOnClickListener(v -> {
            int printWidth = (58 - 10) * 8;
            Bitmap bitmap = BitmapFactory.decodeResource(getResources(), R.mipmap.icod_2);
            bitmap = BitmapUtils.reSize(bitmap, printWidth, bitmap.getHeight() * printWidth / bitmap.getWidth());
            byte[] bmpBytes = PrintImageUtils.parseBmpToByte(bitmap);

            for (PrinterAPI mPrinter : printerArray)
                mPrinter.sendOrder(bmpBytes);
        });

        LinearLayoutManager linearLayoutManager = new LinearLayoutManager(this);
        rlDevice.setLayoutManager(linearLayoutManager);
        rlDevice.addItemDecoration(new DividerItemDecoration(this, DividerItemDecoration.VERTICAL));
        usbDeviceAdapter = new UsbDeviceAdapter(deviceList, new UsbDeviceAdapter.OnItemClick() {
            @Override
            public void onItemClick(int position) {
                USBAPI io = new USBAPI(USBActivity.this, deviceList.get(position));
                PrinterAPI mPrinter = PrinterAPI.getInstance();
                if (mPrinter.connect(io) == PrinterAPI.SUCCESS) {
                    printerArray.add(mPrinter);
                    onToast("OK");
                }
            }
        });
        rlDevice.setAdapter(usbDeviceAdapter);
    }

    @Override
    protected void initData() {
        //注册监听
        if (null == mUsbBroadCastReceiver) {
            IntentFilter intentFilter = new IntentFilter();
            intentFilter.addAction(UsbManager.ACTION_USB_DEVICE_DETACHED);
            intentFilter.addAction(UsbManager.ACTION_USB_DEVICE_ATTACHED);
            mUsbBroadCastReceiver = new UsbBroadCastReceiver();
            registerReceiver(mUsbBroadCastReceiver, intentFilter);
        }

        mUsbManager = (UsbManager) getSystemService(Context.USB_SERVICE);
        HashMap<String, UsbDevice> usbDeviceList = mUsbManager.getDeviceList();
        for (UsbDevice usbDevice : usbDeviceList.values()) {
            deviceList.add(usbDevice);
        }
        usbDeviceAdapter.notifyDataSetChanged();
    }

    //usb 监听 由于权限问题而且sdk内部有超时时间 ,适用情况应该是系统默认usb权限开放或者root 板则使用UsbNativeApi
    private class UsbBroadCastReceiver extends BroadcastReceiver {
        @Override
        public void onReceive(Context context, Intent intent) {
            UsbDevice usbDevice = intent.getParcelableExtra(UsbManager.EXTRA_DEVICE);
            switch (intent.getAction()) {
                case UsbManager.ACTION_USB_DEVICE_ATTACHED:
                    //插USB
                    onToast("插USB: vid=" + usbDevice.getVendorId() + "  pid=" + usbDevice.getProductId());
                    deviceList.add(usbDevice);
                    usbDeviceAdapter.notifyDataSetChanged();
                    break;
                case UsbManager.ACTION_USB_DEVICE_DETACHED:
                    //拔USB
                    onToast("拔USB: vid=" + usbDevice.getVendorId() + "  pid=" + usbDevice.getProductId());
                    for (int i = 0; i < deviceList.size(); i++) {
                        UsbDevice device = deviceList.get(i);
                        if (device.getDeviceName().equals(usbDevice.getDeviceName())) {
                            deviceList.remove(i);
                            usbDeviceAdapter.notifyDataSetChanged();
                        }
                    }
                    break;
            }
        }
    }

    @Override
    protected void onDestroy() {
        super.onDestroy();
        unregisterReceiver(mUsbBroadCastReceiver);
    }
}

